# -*- coding: utf-8 -*-

from framework import LinguistTestBase, main
from libs.classifier import Classifier
from libs.tokenizer import Tokenizer
from libs.samples import DATA

TEST_FILE = "../samples/%s"

class TestClassifier(LinguistTestBase):

    def fixture(self, name):
        return open(TEST_FILE % name).read()

    def test_classify(self):
        db = {}
        Classifier.train(db, "Ruby", self.fixture("Ruby/foo.rb"))
        Classifier.train(db, "Objective-C", self.fixture("Objective-C/Foo.h"))
        Classifier.train(db, "Objective-C", self.fixture("Objective-C/Foo.m"))

        rs = Classifier.classify(db, self.fixture("Objective-C/hello.m"))
        assert "Objective-C" == rs[0][0]

        tokens = Tokenizer.tokenize(self.fixture("Objective-C/hello.m"))
        rs = Classifier.classify(db, tokens)
        assert "Objective-C" == rs[0][0]

    def test_restricted_classify(self):
        db = {}
        Classifier.train(db, "Ruby", self.fixture("Ruby/foo.rb"))
        Classifier.train(db, "Objective-C", self.fixture("Objective-C/Foo.h"))
        Classifier.train(db, "Objective-C", self.fixture("Objective-C/Foo.m"))

        rs = Classifier.classify(db, self.fixture("Objective-C/hello.m"), ["Objective-C"])
        assert "Objective-C" == rs[0][0]

        rs = Classifier.classify(db, self.fixture("Objective-C/hello.m"), ["Ruby"])
        assert "Ruby" == rs[0][0]

    def test_instance_classify_empty(self):
        rs = Classifier.classify(DATA, "")
        r = rs[0]
        assert r[1] < 0.5, str(r)

    def test_instance_classify_none(self):
        assert [] == Classifier.classify(DATA, None)

    def test_classify_ambiguous_languages(self):
        #TODO
        """
        Samples.each do |sample|
          language  = Linguist::Language.find_by_name(sample[:language])
          languages = Language.find_by_filename(sample[:path]).map(&:name)
          next unless languages.length > 1

          results = Classifier.classify(Samples::DATA, File.read(sample[:path]), languages)
          assert_equal language.name, results.first[0], "#{sample[:path]}\n#{results.inspect}"
        end
        """

if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

from pygments.lexers import find_lexer_class
from framework import LinguistTestBase, main
from libs.language import Language


colorize = """<div class="highlight"><pre><span class="k">def</span> <span class="nf">foo</span>
  <span class="s1">&#39;foo&#39;</span>
<span class="k">end</span>
</pre></div>
"""


class TestLanguage(LinguistTestBase):

    def test_lexer(self):
        assert find_lexer_class('ActionScript 3') == Language['ActionScript'].lexer
        assert find_lexer_class('Bash') == Language['Gentoo Ebuild'].lexer
        assert find_lexer_class('Bash') == Language['Gentoo Eclass'].lexer
        assert find_lexer_class('Bash') == Language['Shell'].lexer
        assert find_lexer_class('C') == Language['OpenCL'].lexer
        assert find_lexer_class('C') == Language['XS'].lexer
        assert find_lexer_class('C++') == Language['C++'].lexer
        assert find_lexer_class('Coldfusion HTML') == Language['ColdFusion'].lexer
        assert find_lexer_class('Coq') == Language['Coq'].lexer
        assert find_lexer_class('FSharp') == Language['F#'].lexer
        assert find_lexer_class('FSharp') == Language['F#'].lexer
        assert find_lexer_class('Fortran') == Language['FORTRAN'].lexer
        assert find_lexer_class('Gherkin') == Language['Cucumber'].lexer
        assert find_lexer_class('Groovy') == Language['Groovy'].lexer
        assert find_lexer_class('HTML') == Language['HTML'].lexer
        assert find_lexer_class('HTML+Django/Jinja') == Language['HTML+Django'].lexer
        assert find_lexer_class('HTML+PHP') == Language['HTML+PHP'].lexer
        assert find_lexer_class('HTTP') == Language['HTTP'].lexer
        assert find_lexer_class('JSON') == Language['JSON'].lexer
        assert find_lexer_class('Java') == Language['ChucK'].lexer
        assert find_lexer_class('Java') == Language['Java'].lexer
        assert find_lexer_class('JavaScript') == Language['JavaScript'].lexer
        assert find_lexer_class('MOOCode') == Language['Moocode'].lexer
        assert find_lexer_class('MuPAD') == Language['mupad'].lexer
        assert find_lexer_class('NASM') == Language['Assembly'].lexer
        assert find_lexer_class('OCaml') == Language['OCaml'].lexer
        assert find_lexer_class('Ooc') == Language['ooc'].lexer
        assert find_lexer_class('OpenEdge ABL') == Language['OpenEdge ABL'].lexer
        assert find_lexer_class('REBOL') == Language['Rebol'].lexer
        assert find_lexer_class('RHTML') == Language['HTML+ERB'].lexer
        assert find_lexer_class('RHTML') == Language['RHTML'].lexer
        assert find_lexer_class('Ruby') == Language['Mirah'].lexer
        assert find_lexer_class('Ruby') == Language['Ruby'].lexer
        assert find_lexer_class('S') == Language['R'].lexer
        assert find_lexer_class('Scheme') == Language['Emacs Lisp'].lexer
        assert find_lexer_class('Scheme') == Language['Nu'].lexer
        assert find_lexer_class('Racket') == Language['Racket'].lexer
        assert find_lexer_class('Scheme') == Language['Scheme'].lexer
        assert find_lexer_class('Standard ML') == Language['Standard ML'].lexer
        assert find_lexer_class('TeX') == Language['TeX'].lexer
        assert find_lexer_class('verilog') == Language['Verilog'].lexer
        assert find_lexer_class('XSLT') == Language['XSLT'].lexer
        assert find_lexer_class('aspx-vb') == Language['ASP'].lexer
        # assert find_lexer_class('haXe') == Language['Haxe'].lexer
        assert find_lexer_class('reStructuredText') == Language['reStructuredText'].lexer

    def test_find_by_alias(self):
        assert Language['ASP'] == Language.find_by_alias('asp')
        assert Language['ASP'] == Language.find_by_alias('aspx')
        assert Language['ASP'] == Language.find_by_alias('aspx-vb')
        assert Language['ActionScript'] == Language.find_by_alias('as3')
        assert Language['ApacheConf'] == Language.find_by_alias('apache')
        assert Language['Assembly'] == Language.find_by_alias('nasm')
        assert Language['Batchfile'] == Language.find_by_alias('bat')
        assert Language['C#'] == Language.find_by_alias('c#')
        assert Language['C#'] == Language.find_by_alias('csharp')
        assert Language['C'] == Language.find_by_alias('c')
        assert Language['C++'] == Language.find_by_alias('c++')
        assert Language['C++'] == Language.find_by_alias('cpp')
        assert Language['CoffeeScript'] == Language.find_by_alias('coffee')
        assert Language['CoffeeScript'] == Language.find_by_alias('coffee-script')
        assert Language['ColdFusion'] == Language.find_by_alias('cfm')
        assert Language['Common Lisp'] == Language.find_by_alias('common-lisp')
        assert Language['Common Lisp'] == Language.find_by_alias('lisp')
        assert Language['Darcs Patch'] == Language.find_by_alias('dpatch')
        assert Language['Dart'] == Language.find_by_alias('dart')
        assert Language['Emacs Lisp'] == Language.find_by_alias('elisp')
        assert Language['Emacs Lisp'] == Language.find_by_alias('emacs')
        assert Language['Emacs Lisp'] == Language.find_by_alias('emacs-lisp')
        assert Language['Gettext Catalog'] == Language.find_by_alias('pot')
        assert Language['HTML'] == Language.find_by_alias('html')
        assert Language['HTML'] == Language.find_by_alias('xhtml')
        assert Language['HTML+ERB'] == Language.find_by_alias('html+erb')
        assert Language['HTML+ERB'] == Language.find_by_alias('erb')
        assert Language['IRC log'] == Language.find_by_alias('irc')
        assert Language['JSON'] == Language.find_by_alias('json')
        assert Language['Java Server Pages'] == Language.find_by_alias('jsp')
        assert Language['Java'] == Language.find_by_alias('java')
        assert Language['JavaScript'] == Language.find_by_alias('javascript')
        assert Language['JavaScript'] == Language.find_by_alias('js')
        assert Language['Literate Haskell'] == Language.find_by_alias('lhs')
        assert Language['Literate Haskell'] == Language.find_by_alias('literate-haskell')
        assert Language['Objective-C'] == Language.find_by_alias('objc')
        assert Language['OpenEdge ABL'] == Language.find_by_alias('openedge')
        assert Language['OpenEdge ABL'] == Language.find_by_alias('progress')
        assert Language['OpenEdge ABL'] == Language.find_by_alias('abl')
        assert Language['Parrot Internal Representation'] == Language.find_by_alias('pir')
        assert Language['PowerShell'] == Language.find_by_alias('posh')
        assert Language['Puppet'] == Language.find_by_alias('puppet')
        assert Language['Pure Data'] == Language.find_by_alias('pure-data')
        assert Language['Raw token data'] == Language.find_by_alias('raw')
        assert Language['Ruby'] == Language.find_by_alias('rb')
        assert Language['Ruby'] == Language.find_by_alias('ruby')
        assert Language['Scheme'] == Language.find_by_alias('scheme')
        assert Language['Shell'] == Language.find_by_alias('bash')
        assert Language['Shell'] == Language.find_by_alias('sh')
        assert Language['Shell'] == Language.find_by_alias('shell')
        assert Language['Shell'] == Language.find_by_alias('zsh')
        assert Language['TeX'] == Language.find_by_alias('tex')
        assert Language['TypeScript'] == Language.find_by_alias('ts')
        assert Language['VimL'] == Language.find_by_alias('vim')
        assert Language['VimL'] == Language.find_by_alias('viml')
        assert Language['reStructuredText'] == Language.find_by_alias('rst')
        assert Language['YAML'] == Language.find_by_alias('yml')

    def test_groups(self):
        # Test a couple identity cases
        assert Language['Perl'] == Language['Perl'].group
        assert Language['Python'] == Language['Python'].group
        assert Language['Ruby'] == Language['Ruby'].group

        # Test a few special groups
        assert Language['Assembly'] == Language['GAS'].group
        assert Language['C'] == Language['OpenCL'].group
        assert Language['Haskell'] == Language['Literate Haskell'].group
        assert Language['Java'] == Language['Java Server Pages'].group
        assert Language['Python'] == Language['Cython'].group
        assert Language['Python'] == Language['NumPy'].group
        assert Language['Shell'] == Language['Batchfile'].group
        assert Language['Shell'] == Language['Gentoo Ebuild'].group
        assert Language['Shell'] == Language['Gentoo Eclass'].group
        assert Language['Shell'] == Language['Tcsh'].group

        # Ensure everyone has a group
        for language in Language.all():
            assert language.group, "%s has no group" % language

    def test_search_term(self):
        assert 'perl' == Language['Perl'].search_term
        assert 'python' == Language['Python'].search_term
        assert 'ruby' == Language['Ruby'].search_term
        assert 'common-lisp' == Language['Common Lisp'].search_term
        assert 'html+erb' == Language['HTML+ERB'].search_term
        assert 'max/msp' == Language['Max'].search_term
        assert 'puppet' == Language['Puppet'].search_term
        assert 'pure-data' == Language['Pure Data'].search_term

        assert 'aspx-vb' == Language['ASP'].search_term
        assert 'as3' == Language['ActionScript'].search_term
        assert 'nasm' == Language['Assembly'].search_term
        assert 'bat' == Language['Batchfile'].search_term
        assert 'csharp' == Language['C#'].search_term
        assert 'cpp' == Language['C++'].search_term
        assert 'cfm' == Language['ColdFusion'].search_term
        assert 'dpatch' == Language['Darcs Patch'].search_term
        assert 'fsharp' == Language['F#'].search_term
        assert 'pot' == Language['Gettext Catalog'].search_term
        assert 'irc' == Language['IRC log'].search_term
        assert 'lhs' == Language['Literate Haskell'].search_term
        assert 'ruby' == Language['Mirah'].search_term
        assert 'raw' == Language['Raw token data'].search_term
        assert 'bash' == Language['Shell'].search_term
        assert 'vim' == Language['VimL'].search_term
        assert 'jsp' == Language['Java Server Pages'].search_term
        assert 'rst' == Language['reStructuredText'].search_term

    def test_popular(self):
        assert Language['Ruby'].is_popular
        assert Language['Perl'].is_popular
        assert Language['Python'].is_popular
        assert Language['Assembly'].is_unpopular
        assert Language['Brainfuck'].is_unpopular

    def test_programming(self):
        assert 'programming' == Language['JavaScript'].type
        assert 'programming' == Language['Perl'].type
        assert 'programming' == Language['PowerShell'].type
        assert 'programming' == Language['Python'].type
        assert 'programming' == Language['Ruby'].type
        assert 'programming' == Language['TypeScript'].type

    def test_markup(self):
        assert 'markup' == Language['HTML'].type

    def test_data(self):
        assert 'data' == Language['YAML'].type

    def test_other(self):
        assert None == Language['Brainfuck'].type
        assert None == Language['Makefile'].type

    def test_searchable(self):
        assert True == Language['Ruby'].is_searchable
        assert False == Language['Gettext Catalog'].is_searchable
        assert False == Language['SQL'].is_searchable

    def test_find_by_name(self):
        assert Language['Ruby'] == Language.name_index['Ruby']

    def test_find_all_by_name(self):
        for language in Language.all():
            assert language == Language[language.name]
            assert language == Language.name_index[language.name]

    def test_find_all_by_alias(self):
        for language in Language.all():
            for name in language.aliases:
                assert language == Language.find_by_alias(name)

    def test_find_by_filename(self):
        assert [Language['Shell']] == Language.find_by_filename('PKGBUILD')
        assert [Language['Ruby']] == Language.find_by_filename('foo.rb')
        assert [Language['Ruby']] == Language.find_by_filename('foo/bar.rb')
        assert [Language['Ruby']] == Language.find_by_filename('Rakefile')
        assert [Language['Ruby']] == Language.find_by_filename('PKGBUILD.rb')
        assert Language['ApacheConf'] == Language.find_by_filename('httpd.conf')[0]
        assert [Language['ApacheConf']] == Language.find_by_filename('.htaccess')
        assert Language['Nginx'] == Language.find_by_filename('nginx.conf')[0]
        assert ['C', 'C++', 'Objective-C'] == sorted(map(lambda l: l.name, Language.find_by_filename('foo.h')))
        assert [] == Language.find_by_filename('rb')
        assert [] == Language.find_by_filename('.rb')
        assert [] == Language.find_by_filename('.nkt')
        assert [Language['Shell']] == Language.find_by_filename('.bashrc')
        assert [Language['Shell']] == Language.find_by_filename('bash_profile')
        assert [Language['Shell']] == Language.find_by_filename('.zshrc')
        assert [Language['Clojure']] == Language.find_by_filename('riemann.config')
        assert [Language['HTML+Django']] == Language.find_by_filename('index.jinja')

    def test_find(self):
        assert 'Ruby' == Language['Ruby'].name
        assert 'Ruby' == Language['ruby'].name
        assert 'C++' == Language['C++'].name
        assert 'C++' == Language['c++'].name
        assert 'C++' == Language['cpp'].name
        assert 'C#' == Language['C#'].name
        assert 'C#' == Language['c#'].name
        assert 'C#' == Language['csharp'].name
        assert None == Language['defunkt']

    def test_name(self):
        assert 'Perl' == Language.name_index['Perl'].name
        assert 'Python' == Language.name_index['Python'].name
        assert 'Ruby' == Language.name_index['Ruby'].name

    def test_escaped_name(self):
        assert 'C' == Language['C'].escaped_name
        assert 'C%23' == Language['C#'].escaped_name
        assert 'C%2B%2B' == Language['C++'].escaped_name
        assert 'Objective-C' == Language['Objective-C'].escaped_name
        assert 'Common%20Lisp' == Language['Common Lisp'].escaped_name

    def test_error_without_name(self):
        self.assertRaises(KeyError, Language, {})

    def test_color(self):
        assert '#701516' == Language['Ruby'].color
        assert '#3581ba' == Language['Python'].color
        assert '#f15501' == Language['JavaScript'].color
        assert '#31859c' == Language['TypeScript'].color

    def test_colors(self):
        Language['Ruby'] in Language.colors()
        Language['Python'] in Language.colors()

    def test_ace_mode(self):
        assert 'c_cpp' == Language['C++'].ace_mode
        assert 'coffee' == Language['CoffeeScript'].ace_mode
        assert 'csharp' == Language['C#'].ace_mode
        assert 'css' == Language['CSS'].ace_mode
        assert 'javascript' == Language['JavaScript'].ace_mode

    def test_ace_modes(self):
        assert Language['Ruby'] in Language.ace_modes()
        assert Language['FORTRAN'] not in Language.ace_modes()

    def test_wrap(self):
        assert False == Language['C'].wrap
        assert True == Language['Markdown'].wrap

    def test_extensions(self):
        assert '.pl' in Language['Perl'].extensions
        assert '.py' in Language['Python'].extensions
        assert '.rb' in Language['Ruby'].extensions

    def test_primary_extension(self):
        assert '.pl' == Language['Perl'].primary_extension
        assert '.py' == Language['Python'].primary_extension
        assert '.rb' == Language['Ruby'].primary_extension
        assert '.js' == Language['JavaScript'].primary_extension
        assert '.coffee' == Language['CoffeeScript'].primary_extension
        assert '.t' == Language['Turing'].primary_extension
        assert '.ts' == Language['TypeScript'].primary_extension

        # This is a nasty requirement, but theres some code in GitHub that
        # expects this. Really want to drop this.
        for language in Language.all():
            assert language.primary_extension, "%s has no primary extension" % language

    def test_eql(self):
        assert Language['Ruby'] == Language['Ruby']
        assert Language['Ruby'] != Language['Python']

    def test_colorize(self):
        assert colorize == Language['Ruby'].colorize("def foo\n  'foo'\nend\n")


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

import os
import sys

TEST_DIR = os.path.dirname(os.path.realpath(__file__))
ROOT_DIR = os.path.dirname(TEST_DIR)
sys.path.insert(0, ROOT_DIR)

from unittest import main, TestSuite, findTestCases

def get_test_module_names():
    file_names = os.listdir(os.curdir)
    for fn in file_names:
        if fn.startswith('test') and fn.endswith('.py'):
            yield 'tests.' + fn[:-3]

def suite():
    alltests = TestSuite()

    for module_name in get_test_module_names():
        module = __import__(module_name, fromlist=[module_name])
        alltests.addTest(findTestCases(module))

    return alltests


if __name__ == '__main__':
    main(defaultTest='suite')



# -*- coding: utf-8 -*-

from framework import LinguistTestBase, main
from libs.samples import DATA


class TestSamples(LinguistTestBase):

    def test_verify(self):
        data = DATA
        assert data['languages_total'] == sum(data['languages'].values())
        assert data['tokens_total'] == sum(data['language_tokens'].values())
        assert data['tokens_total'] == sum(reduce(lambda x, y: x + y,
                                                  [token.values() for token in data['tokens'].values()]))

if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

from os import stat
from blob_helper import BlobHelper


class FileBlob(BlobHelper):
    """
    A FileBlob is a wrapper around a File object to make it quack
    like a Grit::Blob. It provides the basic interface: `name`,
    `data`, and `size`.
    """

    def __init__(self, path, base_path=None):
        """
        Public: Initialize a new FileBlob from a path

          path      - A path String that exists on the file system.
          base_path - Optional base to relativize the path

        Returns a FileBlob.


        """
        self.path = path

        """
        Public: name

        Examples

          FileBlob.new("/path/to/linguist/lib/linguist.rb").name
          # =>  "/path/to/linguist/lib/linguist.rb"

          FileBlob.new("/path/to/linguist/lib/linguist.rb",
                       "/path/to/linguist").name
          # =>  "lib/linguist.rb"

        Returns a String
        """
        if base_path:
            base_path = base_path.rstrip('/')
        self.name = base_path and path.replace('%s/' % base_path, '', 1) or path

    def __repr__(self):
        return '<FileBlob name:%s>' % self.name

    @property
    def stat(self):
        return stat(self.path)

    @property
    def mode(self):
        """
        Public: Read file permissions

        Returns a String like '100644'
        """
        mode = self.stat.st_mode
        return oct(mode)

    @property
    def data(self):
        """
        Public: Read file contents.

        Returns a String.
        """
        if hasattr(self, '_data'):
            return self._data
        self._data = file(self.path).read()
        return self._data

    @property
    def size(self):
        """
        Public: Get byte size

        Returns an Integer.
        """
        return self.stat.st_size



# -*- coding: utf-8 -*-
import os
import sys
import math
from functools import partial

is_py27 = sys.version_info >= (2, 7)
if is_py27:
    from collections import Counter
from tokenizer import Tokenizer


class Classifier(object):
    """ Language bayesian classifier. """

    verbosity = int(os.environ.get('LINGUIST_DEBUG', '0'))

    @classmethod
    def train(cls, db, language, data):
        """
        Set LINGUIST_DEBUG=1 or =2 to see probabilities per-token,
        per-language.  See also dump_all_tokens, below.

        Public: Train classifier that data is a certain language.

          db       - Hash classifier database object
          language - String language of data
          data     - String contents of file

          Examples

            Classifier.train(db, 'Ruby', "def hello; end")

          Returns nothing.
        """
        tokens = Tokenizer.tokenize(data)
        db['tokens_total'] = db.get('tokens_total', 0)
        db['languages_total'] = db.get('languages_total', 0)
        db['tokens'] = db.get('tokens', {})
        db['language_tokens'] = db.get('language_tokens', {})
        db['languages'] = db.get('languages', {})

        for token in tokens:
            db['tokens'][language] = db['tokens'].get(language, {})
            db['tokens'][language][token] = db['tokens'][language].get(token, 0)
            db['tokens'][language][token] += 1
            db['language_tokens'][language] = db['language_tokens'].get(language, 0)
            db['language_tokens'][language] += 1
            db['tokens_total'] += 1

        db['languages'][language] = db['languages'].get(language, 0)
        db['languages'][language] += 1
        db['languages_total'] += 1

    def __init__(self, db={}):
        self.tokens = db.get('tokens')
        self.tokens_total = db.get('tokens_total')
        self.languages = db.get('languages')
        self.languages_total = db.get('languages_total')
        self.language_tokens = db.get('language_tokens')

    def __repr__(self):
        return '<Classifier>'

    @classmethod
    def classify(cls, db, tokens, languages=[]):
        """
        Public: Guess language of data.

        db        - Hash of classifer tokens database.
        data      - Array of tokens or String data to analyze.
        languages - Array of language name Strings to restrict to.

        Examples

          Classifier.classify(db, "def hello; end")
          # => [ 'Ruby', 0.90], ['Python', 0.2], ... ]

        Returns sorted Array of result pairs. Each pair contains the
        String language name and a Float score.
        """
        languages = languages or db.get('languages', {}).keys()
        return cls(db)._classify(tokens, languages)

    def _classify(self, tokens, languages):
        """
        Internal: Guess language of data

        data      - Array of tokens or String data to analyze.
        languages - Array of language name Strings to restrict to.

        Returns sorted Array of result pairs. Each pair contains the
        String language name and a Float score.
        """
        if tokens is None:
            return []

        if isinstance(tokens, basestring):
            tokens = Tokenizer.tokenize(tokens)

        scores = {}
        if self.verbosity >= 2:
            self.dump_all_tokens(tokens, languages)
        for language in languages:
            scores[language] = self.tokens_probability(tokens, language) + self.language_probability(language)
            if self.verbosity >= 1:
                print '%10s = %10.3f + %7.3f = %10.3f\n' % (language,
                                                            self.tokens_probability(tokens, language),
                                                            self.language_probability(language),
                                                            scores[language])
        return sorted(scores.iteritems(), key=lambda t: t[1], reverse=True)

    def tokens_probability(self, tokens, language):
        """
        Internal: Probably of set of tokens in a language occuring - P(D | C)

        tokens   - Array of String tokens.
        language - Language to check.

        Returns Float between 0.0 and 1.0.
        """
        token_probability = partial(self.token_probability, language=language)
        return reduce(lambda x, y: x + math.log(token_probability(y)), tokens, 0.0)

    def token_probability(self, token, language=''):
        """
        Internal: Probably of token in language occuring - P(F | C)

        token    - String token.
        language - Language to check.

        Returns Float between 0.0 and 1.0.
        """
        probability = float(self.tokens.get(language, {}).get(token, 0))
        if probability == 0.0:
            return 1 / float(self.tokens_total)
        else:
            return probability / float(self.language_tokens[language])

    def language_probability(self, language):
        """
        Internal: Probably of a language occuring - P(C)

        language - Language to check.

        Returns Float between 0.0 and 1.0.
        """
        return math.log(float(self.languages[language]) / float(self.languages_total))

    def dump_all_tokens(self, tokens, languages):
        """
        Internal: show a table of probabilities for each <token,language> pair.

        The number in each table entry is the number of "points" that each
        token contributes toward the belief that the file under test is a
        particular language.  Points are additive.

        Points are the number of times a token appears in the file, times
        how much more likely (log of probability ratio) that token is to
        appear in one language vs.  the least-likely language.  Dashes
        indicate the least-likely language (and zero points) for each token.
        """
        maxlen = max([len(token) for token in tokens])

        print '%ss' % maxlen
        print '    #' + ''.join(['%10s' for lang in languages])

        if not is_py27:
            return

        tokmap = Counter(tokens)
        for tok, count in tokmap.most_common():
            arr = [(lang, self.token_probability(tok, lang)) for lang in languages]
            minlen = min([n for lang, n in arr])
            minlog = math.log(minlen)

            if not reduce(lambda x, y: x and y[1] == arr[0][1], arr, True):
                print '%*s%5d' % (maxlen, tok, count)

                for ent in arr:
                    if ent[1] == minlen:
                        print '         -'
                    else:
                        print '%10.3f' % (math.log(ent[1]) - minlog)



# -*- coding: utf-8 -*-

import re
from os.path import splitext

XCODE_PROJECT_EXT_NAMES = ('.xib', '.nib', '.storyboard', '.pbxproj',
                           '.xcworkspacedata', '.xcuserstate')


class Generated(object):

    def __init__(self, name, data):
        self.name = name
        self.ext_name = splitext(self.name)[1].lower()
        self._data = data

    def __repr__(self):
        return '<Generated name:%s>' % self.name

    @classmethod
    def is_generated(cls, name, data):
        return cls(name, data)._is_generated

    @property
    def data(self):
        if hasattr(self, 'real_data'):
            return self.real_data
        self.real_data = self._data() if callable(self._data) else self._data
        return self.real_data

    @property
    def lines(self):
        return self.data and self.data.split("\n", -1) or []

    @property
    def _is_generated(self):
        return any((self.name == "Gemfile.lock",
                    self.is_minified_files,
                    self.is_compiled_coffeescript,
                    self.is_xcode_project_file,
                    self.is_generated_parser,
                    self.is_generated_net_docfile,
                    self.is_generated_net_designer_file,
                    self.is_generated_protocol_buffer,
                    self.is_generated_jni_header))

    @property
    def is_xcode_project_file(self):
        """
        Internal: Is the blob an XCode project file?

        Generated if the file extension is an XCode project
        file extension.

        Returns True of False.
        """
        return self.ext_name in XCODE_PROJECT_EXT_NAMES

    @property
    def is_minified_files(self):
        """
        Internal: Is the blob minified files?

        Consider a file minified if it contains more than 5% spaces.
        Currently, only JS and CSS files are detected by this method.

        Returns True or False.
        """
        if self.ext_name not in ('.js', '.css'):
            return False
        if self.data and len(self.data) > 200:
            count_space = sum([1 for _ in self.data if _ <= ' '])
            return (count_space / float(len(self.data))) < 0.05
        return False

    @property
    def is_compiled_coffeescript(self):
        """
        Internal: Is the blob of JS generated by CoffeeScript?

        CoffeeScript is meant to output JS that would be difficult to
        tell if it was generated or not. Look for a number of patterns
        output by the CS compiler.

        Return True or False
        """
        if self.ext_name != ".js":
            return False

        lines = self.lines

        # CoffeeScript generated by > 1.2 include a comment on the first line
        if len(lines) > 0 \
                and re.compile('^\/\/ Generated by ', re.DOTALL).search(lines[0]):
            return True

        if len(lines) < 3:
            return False

        if len(lines) > 2 \
                and lines[0] == '(function() {'  \
                and lines[-2] == '}).call(this);' \
                and lines[-1] == '':
            # First line is module closure opening
            # Second to last line closes module closure
            # Last line is blank
            score = 0
            count_keys = lambda r, s: len(re.compile(r, re.DOTALL).findall(s))
            for line in lines:
                if re.compile('var ', re.DOTALL).search(line):
                    # Underscored temp vars are likely to be Coffee
                    score += 1 * count_keys('(_fn|_i|_len|_ref|_results)', line)
                    # bind and extend functions are very Coffee specific
                    score += 3 * count_keys('(__bind|__extends|__hasProp|__indexOf|__slice)', line)
            return score > 3
        return False

    @property
    def is_generated_net_docfile(self):
        """
        Internal: Is this a generated documentation file for a .NET assembly?

        .NET developers often check in the XML Intellisense file along with an
        assembly - however, these don't have a special extension, so we have to
        dig into the contents to determine if it's a docfile. Luckily, these files
        are extremely structured, so recognizing them is easy.

        Returns True or False
          return false unless extname.downcase == ".xml"
          return false unless lines.count > 3
        """
        if self.ext_name != ".xml":
            return False

        lines = self.lines

        if len(lines) < 3:
            return False

        """
        .NET Docfiles always open with <doc> and their first tag is an
        <assembly> tag
        """
        return '<doc>' in lines[1] \
               and '<assembly>' in lines[2] \
               and '</doc>' in lines[-2]

    @property
    def is_generated_net_designer_file(self):
        """
        Internal: Is this a codegen file for a .NET project?

        Visual Studio often uses code generation to generate partial
        classes, and these files can be quite unwieldy. Let's hide them.

        Returns true or false
        """
        return self.name.lower().endswith('.designer.cs')

    @property
    def is_generated_parser(self):
        """
        Internal: Is the blob of JS a parser generated by PEG.js?

        PEG.js-generated parsers are not meant to be consumed by humans.

        Return True or False
        """
        if self.ext_name != ".js":
            return False

        # PEG.js-generated parsers include a comment near the top  of the file
        # that marks them as such.
        lines = self.lines
        if len(lines) < 5:
            return False

        if re.compile('^(?:[^\/]|\/[^\*])*\/\*(?:[^\*]|\*[^\/])*Generated by PEG.js', re.DOTALL).search(''.join(lines[0:5])):
            return True
        return False

    @property
    def is_generated_protocol_buffer(self):
        """
        Internal: Is the blob a C++, Java or Python source file generated by the
        Protocol Buffer compiler?

        Returns true of false.
        """
        if self.ext_name not in ('.py', '.java', '.h', '.cc', '.cpp'):
            return False

        if len(self.lines) < 1:
            return False

        return 'Generated by the protocol buffer compiler.  DO NOT EDIT!' in self.lines[0]

    @property
    def is_generated_jni_header(self):
        """
        Internal: Is the blob a C/C header generated by the Java JNI tool javah?

        Returns true of false.
        """
        if self.ext_name != '.h':
            return False
        if len(self.lines) < 2:
            return False

        return all(("/* DO NOT EDIT THIS FILE - it is machine generated */" in self.lines[0],
                    "#include <jni.h>" in self.lines[1]))

    @property
    def is_node_modules(self):
        """
        node_modules/ can contain large amounts of files, in general not meant
        for humans in pull requests.

        Returns true or false.
        """
        return bool(re.compile('node_modules/').search(self.name))



# -*- coding: utf-8 -*-
from re import compile, escape

from scanner import StringScanner, StringRegexp

"""
Generic programming language tokenizer.

Tokens are designed for use in the language bayes classifier.
It strips any data strings or comments and preserves significant
language symbols.
"""

# Read up to 100KB
BYTE_LIMIT = 100000

# Start state on token, ignore anything till the next newline
SINGLE_LINE_COMMENTS = [
    '//',  # C
    '#',   # Python, Ruby
    '%',   # Tex
]

# Start state on opening token, ignore anything until the closing
# token is reached.
MULTI_LINE_COMMENTS = [
    [r'/*', r'*/'],     # C
    [r'<!--', r'-->'],  # XML
    [r'{-', r'-}'],     # Haskell
    [r'(*', r'*)'],     # Coq
    [r'"""', r'"""'],   # Python
    [r"'''", r"'''"],   # Python
]

MULTI_LINE_COMMENT_DICT = dict([(s, StringRegexp(escape(e)))
                                for s, e in MULTI_LINE_COMMENTS])

START_SINGLE_LINE_COMMENT = StringRegexp('|'.join(map(lambda c: '\s*%s ' % escape(c), SINGLE_LINE_COMMENTS)))
START_MULTI_LINE_COMMENT = StringRegexp('|'.join(map(lambda c: escape(c[0]), MULTI_LINE_COMMENTS)))


REGEX_SHEBANG = StringRegexp(r'^#!.+')
REGEX_BOL = StringRegexp(r'\n|\Z')
REGEX_DOUBLE_QUOTE = StringRegexp(r'"')
REGEX_SINGLE_QUOTE = StringRegexp(r"'")
REGEX_DOUBLE_END_QUOTE = StringRegexp(r'[^\\]"')
REGEX_SINGLE_END_QUOTE = StringRegexp(r"[^\\]'")
REGEX_NUMBER_LITERALS = StringRegexp(r'(0x)?\d(\d|\.)*')
REGEX_SGML = StringRegexp(r'<[^\s<>][^<>]*>')
REGEX_COMMON_PUNCTUATION = StringRegexp(r';|\{|\}|\(|\)|\[|\]')
REGEX_REGULAR_TOKEN = StringRegexp(r'[\w\.@#\/\*]+')
REGEX_COMMON_OPERATORS = StringRegexp(r'<<?|\+|\-|\*|\/|%|&&?|\|\|?')
REGEX_EMIT_START_TOKEN = StringRegexp(r'<\/?[^\s>]+')
REGEX_EMIT_TRAILING = StringRegexp(r'\w+=')
REGEX_EMIT_WORD = StringRegexp(r'\w+')
REGEX_EMIT_END_TAG = StringRegexp('>')

REGEX_SHEBANG_FULL = StringRegexp(r'^#!\s*\S+')
REGEX_SHEBANG_WHITESPACE = StringRegexp(r'\s+')
REGEX_SHEBANG_NON_WHITESPACE = StringRegexp(r'\S+')


class Tokenizer(object):

    def __repr__(self):
        return '<Tokenizer>'

    @classmethod
    def tokenize(cls, data):
        """
        Public: Extract tokens from data

        data - String to tokenize

        Returns Array of token Strings.
        """
        return cls().extract_tokens(data)

    def extract_tokens(self, data):
        """
        Internal: Extract generic tokens from data.

        data - String to scan.

        Examples

          extract_tokens("printf('Hello')")
          # => ['printf', '(', ')']

        Returns Array of token Strings.
        """
        s = StringScanner(data)
        tokens = []
        while not s.is_eos:
            if s.pos >= BYTE_LIMIT:
                break
            token = s.scan(REGEX_SHEBANG)
            if token:
                name = self.extract_shebang(token)
                if name:
                    tokens.append('SHEBANG#!%s' % name)
                continue

            # Single line comment
            if s.is_bol and s.scan(START_SINGLE_LINE_COMMENT):
                s.skip_until(REGEX_BOL)
                continue

            # Multiline comments
            token = s.scan(START_MULTI_LINE_COMMENT)
            if token:
                close_token = MULTI_LINE_COMMENT_DICT[token]
                s.skip_until(close_token)
                continue

            # Skip single or double quoted strings
            if s.scan(REGEX_DOUBLE_QUOTE):
                if s.peek(1) == '"':
                    s.getch
                else:
                    s.skip_until(REGEX_DOUBLE_END_QUOTE)
                continue
            if s.scan(REGEX_SINGLE_QUOTE):
                if s.peek(1) == "'":
                    s.getch
                else:
                    s.skip_until(REGEX_SINGLE_END_QUOTE)
                continue

            # Skip number literals
            if s.scan(REGEX_NUMBER_LITERALS):
                continue

            # SGML style brackets
            token = s.scan(REGEX_SGML)
            if token:
                for t in self.extract_sgml_tokens(token):
                    tokens.append(t)
                continue

            # Common programming punctuation
            token = s.scan(REGEX_COMMON_PUNCTUATION)
            if token:
                tokens.append(token)
                continue

            # Regular token
            token = s.scan(REGEX_REGULAR_TOKEN)
            if token:
                tokens.append(token)
                continue

            # Common operators
            token = s.scan(REGEX_COMMON_OPERATORS)
            if token:
                tokens.append(token)
                continue

            s.getch
        return tokens

    @classmethod
    def extract_shebang(cls, data):
        """
        Internal: Extract normalized shebang command token.

        Examples

          extract_shebang("#!/usr/bin/ruby")
          # => "ruby"

          extract_shebang("#!/usr/bin/env node")
          # => "node"

        Returns String token or nil it couldn't be parsed.
        """
        s = StringScanner(data)
        path = s.scan(REGEX_SHEBANG_FULL)
        if path:
            script = path.split('/')[-1]
            if script == 'env':
                s.scan(REGEX_SHEBANG_WHITESPACE)
                script = s.scan(REGEX_SHEBANG_NON_WHITESPACE)
            if script:
                script = compile(r'[^\d]+').match(script).group(0)
            return script
        return

    def extract_sgml_tokens(self, data):
        """
        Internal: Extract tokens from inside SGML tag.

        data - SGML tag String.

            Examples

              extract_sgml_tokens("<a href='' class=foo>")
              # => ["<a>", "href="]

        Returns Array of token Strings.
        """
        s = StringScanner(data)
        tokens = []
        append = tokens.append

        while not s.is_eos:
            # Emit start token
            token = s.scan(REGEX_EMIT_START_TOKEN)
            if token:
                append(token + '>')
                continue

            # Emit attributes with trailing =
            token = s.scan(REGEX_EMIT_TRAILING)
            if token:
                append(token)

                # Then skip over attribute value
                if s.scan(REGEX_DOUBLE_QUOTE):
                    s.skip_until(REGEX_DOUBLE_END_QUOTE)
                    continue
                if s.scan(REGEX_SINGLE_QUOTE):
                    s.skip_until(REGEX_SINGLE_END_QUOTE)
                    continue
                s.skip_until(REGEX_EMIT_WORD)
                continue

            # Emit lone attributes
            token = s.scan(REGEX_EMIT_WORD)
            if token:
                append(token)

            # Stop at the end of the tag
            if s.scan(REGEX_EMIT_END_TAG):
                s.terminate
                continue

            s.getch

        return tokens



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages
from linguist import VERSION

setup(name='linguist',
      version=VERSION,
      keywords=('linguist', 'detect', 'programming', 'language'),
      description='Language Savant',
      long_description='Language Savant',
      license='New BSD',

      url='https://github.com/liluo/linguist',
      author='liluo',
      author_email='i@liluo.org',

      packages=find_packages(),
      include_package_data=True,
      platforms='any',
      dependency_links = ['https://github.com/liluo/pygments/tarball/master#egg=Pygments-1.6'],
      install_requires=['PyYAML',
                        'pygments-github-lexers>=0.0.3',
                        'charlockholmes',
                        'mime>=0.0.3',
                        'scanner>=0.0.4'],
      classifiers=[],
      scripts=['bin/pylinguist'])



# -*- coding: utf-8 -*-

from os.path import realpath, dirname, join
from pygments.lexers import find_lexer_class
from framework import LinguistTestBase, main
from libs.file_blob import FileBlob
from libs.samples import Samples

DIR = dirname(dirname(realpath(__file__)))
SAMPLES_PATH = join(DIR, "samples")

colorize = """<div class="highlight"><pre><span class="k">module</span> <span class="nn">Foo</span>
<span class="k">end</span>
</pre></div>
"""

colorize_without_wrapper = """<span class="k">module</span> <span class="nn">Foo</span>
<span class="k">end</span>
"""


class TestFileBob(LinguistTestBase):

    def blob(self, name):
        if not name.startswith('/'):
            name = join(SAMPLES_PATH, name)
        return FileBlob(name, SAMPLES_PATH)

    def script_blob(self, name):
        blob = self.blob(name)
        blob.name = 'script'
        return blob

    def test_name(self):
        assert 'foo.rb' == self.blob('foo.rb').name

    def test_mime_type(self):
        assert 'application/postscript' == self.blob('Binary/octocat.ai').mime_type
        assert 'application/x-ruby' == self.blob("Ruby/grit.rb").mime_type
        assert "application/x-sh" == self.blob("Shell/script.sh").mime_type
        assert "application/xml" == self.blob("XML/bar.xml").mime_type
        assert "audio/ogg" == self.blob("Binary/foo.ogg").mime_type
        assert "text/plain" == self.blob("Text/README").mime_type

    def test_content_type(self):
        assert "application/pdf" == self.blob("Binary/foo.pdf").content_type
        assert "audio/ogg" == self.blob("Binary/foo.ogg").content_type
        assert "image/png" == self.blob("Binary/foo.png").content_type
        assert "text/plain; charset=iso-8859-2" == self.blob("Text/README").content_type

    def test_disposition(self):
        assert "attachment; filename=foo+bar.jar" == self.blob("Binary/foo bar.jar").disposition
        assert "attachment; filename=foo.bin" == self.blob("Binary/foo.bin").disposition
        assert "attachment; filename=linguist.gem" == self.blob("Binary/linguist.gem").disposition
        assert "attachment; filename=octocat.ai" == self.blob("Binary/octocat.ai").disposition
        assert "inline" == self.blob("Text/README").disposition
        assert "inline" == self.blob("Text/foo.txt").disposition
        assert "inline" == self.blob("Ruby/grit.rb").disposition
        assert "inline" == self. blob("Binary/octocat.png").disposition

    def test_data(self):
        assert "module Foo\nend\n" == self.blob("Ruby/foo.rb").data

    def test_lines(self):
        assert ["module Foo", "end", ""] == self.blob("Ruby/foo.rb").lines
        assert ["line 1", "line 2", ""] == self.blob("Text/mac.txt").lines

    def test_size(self):
        assert 15 == self.blob("Ruby/foo.rb").size

    def test_loc(self):
        assert 3 == self.blob("Ruby/foo.rb").loc

    def test_sloc(self):
        assert 2 == self.blob("Ruby/foo.rb").sloc

    def test_encoding(self):
        assert "ISO-8859-2" == self.blob("Text/README").encoding
        assert "ISO-8859-1" == self.blob("Text/dump.sql").encoding
        assert "UTF-8" == self.blob("Text/foo.txt").encoding
        assert None == self.blob("Binary/dog.o").encoding

    def test_binary(self):
        # Large blobs aren't loaded
        large_blob = self.blob("git.exe")
        large_blob._data = None
        assert large_blob.is_binary

        assert self.blob("Binary/git.deb").is_binary
        assert self.blob("Binary/git.exe").is_binary
        assert self.blob("Binary/hello.pbc").is_binary
        assert self.blob("Binary/linguist.gem").is_binary
        assert self.blob("Binary/octocat.ai").is_binary
        assert self.blob("Binary/octocat.png").is_binary
        assert self.blob("Binary/zip").is_binary
        assert not self.blob("Text/README").is_binary
        assert not self.blob("Text/file.txt").is_binary
        assert not self.blob("Ruby/foo.rb").is_binary
        assert not self.blob("Perl/script.pl").is_binary

    def test_text(self):
        assert self.blob("Text/README").is_text
        assert self.blob("Text/dump.sql").is_text
        assert self.blob("Text/file.json").is_text
        assert self.blob("Text/file.txt").is_text
        assert self.blob("Text/md").is_text
        assert self.blob("Shell/script.sh").is_text
        assert self.blob("Text/txt").is_text

    def test_image(self):
        assert self.blob("Binary/octocat.gif").is_image
        assert self.blob("Binary/octocat.jpeg").is_image
        assert self.blob("Binary/octocat.jpg").is_image
        assert self.blob("Binary/octocat.png").is_image
        assert not self.blob("Binary/octocat.ai").is_image
        assert not self.blob("Binary/octocat.psd").is_image

    def test_solid(self):
        assert self.blob("Binary/cube.stl").is_solid
        assert self.blob("Text/cube.stl").is_solid

    def test_viewable(self):
        assert self.blob("Text/README").is_viewable
        assert self.blob("Ruby/foo.rb").is_viewable
        assert self.blob("Perl/script.pl").is_viewable
        assert not self.blob("Binary/linguist.gem").is_viewable
        assert not self.blob("Binary/octocat.ai").is_viewable
        assert not self.blob("Binary/octocat.png").is_viewable

    def test_csv(self):
        assert self.blob("Text/cars.csv").is_csv

    def test_pdf(self):
        assert self.blob("Binary/foo.pdf").is_pdf

    def test_generated(self):
        assert not self.blob("Text/README").is_generated

        # Xcode project files
        assert self.blob("XML/MainMenu.xib").is_generated
        assert self.blob("Binary/MainMenu.nib").is_generated
        assert self.blob("XML/project.pbxproj").is_generated

        # Gemfile.locks
        assert self.blob("Gemfile.lock").is_generated

        # Generated .NET Docfiles
        assert self.blob("XML/net_docfile.xml").is_generated

        # Long line
        assert not self.blob("JavaScript/uglify.js").is_generated

        # Inlined JS, but mostly code
        assert not self.blob("JavaScript/json2_backbone.js").is_generated

        # Minified JS
        assert not self.blob("JavaScript/jquery-1.6.1.js").is_generated
        assert self.blob("JavaScript/jquery-1.6.1.min.js").is_generated
        assert self.blob("JavaScript/jquery-1.4.2.min.js").is_generated

        # CoffeeScript-is_generated JS
        # TODO

        # TypeScript-is_generated JS
        # TODO

        # PEG.js-is_generated parsers
        assert self.blob("JavaScript/parser.js").is_generated

        # These examples are too basic to tell
        assert not self.blob("JavaScript/empty.js").is_generated
        assert not self.blob("JavaScript/hello.js").is_generated

        assert self.blob("JavaScript/intro-old.js").is_generated
        assert self.blob("JavaScript/classes-old.js").is_generated

        assert self.blob("JavaScript/intro.js").is_generated
        assert self.blob("JavaScript/classes.js").is_generated

        # Protocol Buffer generated code
        assert self.blob("C++/protocol-buffer.pb.h").is_generated
        assert self.blob("C++/protocol-buffer.pb.cc").is_generated
        assert self.blob("Java/ProtocolBuffer.java").is_generated
        assert self.blob("Python/protocol_buffer_pb2.py").is_generated

        # Generated JNI
        assert self.blob("C/jni_layer.h").is_generated

        # Minified CSS
        assert not self.blob("CSS/bootstrap.css").is_generated
        assert self.blob("CSS/bootstrap.min.css").is_generated

    def test_vendored(self):
        assert not self.blob("Text/README").is_vendored
        assert not self.blob("ext/extconf.rb").is_vendored

        # Dependencies
        assert self.blob("dependencies/windows/headers/GL/glext.h").is_vendored

        # Node depedencies
        assert  self.blob("node_modules/coffee-script/lib/coffee-script.js").is_vendored

        # Rails vendor/
        assert  self.blob("vendor/plugins/will_paginate/lib/will_paginate.rb").is_vendored

        # 'thirdparty' directory
        assert self.blob("thirdparty/lib/main.c").is_vendored

        # C deps
        assert  self.blob("deps/http_parser/http_parser.c").is_vendored
        assert  self.blob("deps/v8/src/v8.h").is_vendored

        # Debian packaging
        assert  self.blob("debian/cron.d").is_vendored

        # Prototype
        assert not self.blob("public/javascripts/application.js").is_vendored
        assert  self.blob("public/javascripts/prototype.js").is_vendored
        assert  self.blob("public/javascripts/effects.js").is_vendored
        assert  self.blob("public/javascripts/controls.js").is_vendored
        assert  self.blob("public/javascripts/dragdrop.js").is_vendored

        # jQuery
        assert  self.blob("jquery.js").is_vendored
        assert  self.blob("public/javascripts/jquery.js").is_vendored
        assert  self.blob("public/javascripts/jquery.min.js").is_vendored
        assert  self.blob("public/javascripts/jquery-1.7.js").is_vendored
        assert  self.blob("public/javascripts/jquery-1.7.min.js").is_vendored
        assert  self.blob("public/javascripts/jquery-1.5.2.js").is_vendored
        assert  self.blob("public/javascripts/jquery-1.6.1.js").is_vendored
        assert  self.blob("public/javascripts/jquery-1.6.1.min.js").is_vendored
        assert  self.blob("public/javascripts/jquery-1.10.1.js").is_vendored
        assert  self.blob("public/javascripts/jquery-1.10.1.min.js").is_vendored
        assert not self.blob("public/javascripts/jquery.github.menu.js").is_vendored

        # jQuery UI
        assert self.blob("themes/ui-lightness/jquery-ui.css").is_vendored
        assert self.blob("themes/ui-lightness/jquery-ui-1.8.22.custom.css").is_vendored
        assert self.blob("themes/ui-lightness/jquery.ui.accordion.css").is_vendored
        assert self.blob("ui/i18n/jquery.ui.datepicker-ar.js").is_vendored
        assert self.blob("ui/i18n/jquery-ui-i18n.js").is_vendored
        assert self.blob("ui/jquery.effects.blind.js").is_vendored
        assert self.blob("ui/jquery-ui-1.8.22.custom.js").is_vendored
        assert self.blob("ui/jquery-ui-1.8.22.custom.min.js").is_vendored
        assert self.blob("ui/jquery-ui-1.8.22.js").is_vendored
        assert self.blob("ui/jquery-ui-1.8.js").is_vendored
        assert self.blob("ui/jquery-ui.min.js").is_vendored
        assert self.blob("ui/jquery.ui.accordion.js").is_vendored
        assert self.blob("ui/minified/jquery.effects.blind.min.js").is_vendored
        assert self.blob("ui/minified/jquery.ui.accordion.min.js").is_vendored

        # MooTools
        assert  self.blob("public/javascripts/mootools-core-1.3.2-full-compat.js").is_vendored
        assert  self.blob("public/javascripts/mootools-core-1.3.2-full-compat-yc.js").is_vendored

        # Dojo
        assert  self.blob("public/javascripts/dojo.js").is_vendored

        # MochiKit
        assert  self.blob("public/javascripts/MochiKit.js").is_vendored

        # YUI
        assert  self.blob("public/javascripts/yahoo-dom-event.js").is_vendored
        assert  self.blob("public/javascripts/yahoo-min.js").is_vendored
        assert  self.blob("public/javascripts/yuiloader-dom-event.js").is_vendored

        # WYS editors
        assert  self.blob("public/javascripts/ckeditor.js").is_vendored
        assert  self.blob("public/javascripts/tiny_mce.js").is_vendored
        assert  self.blob("public/javascripts/tiny_mce_popup.js").is_vendored
        assert  self.blob("public/javascripts/tiny_mce_src.js").is_vendored

        # Fabric
        assert  self.blob("fabfile.py").is_vendored

        # WAF
        assert  self.blob("waf").is_vendored

        # Visual Studio IntelliSense
        assert  self.blob("Scripts/jquery-1.7-vsdoc.js").is_vendored

        # Microsoft Ajax
        assert  self.blob("Scripts/MicrosoftAjax.debug.js").is_vendored
        assert  self.blob("Scripts/MicrosoftAjax.js").is_vendored
        assert  self.blob("Scripts/MicrosoftMvcAjax.debug.js").is_vendored
        assert  self.blob("Scripts/MicrosoftMvcAjax.js").is_vendored
        assert  self.blob("Scripts/MicrosoftMvcValidation.debug.js").is_vendored
        assert  self.blob("Scripts/MicrosoftMvcValidation.js").is_vendored

        # jQuery validation plugin (MS bundles this with asp.net mvc)
        assert  self.blob("Scripts/jquery.validate.js").is_vendored
        assert  self.blob("Scripts/jquery.validate.min.js").is_vendored
        assert  self.blob("Scripts/jquery.validate.unobtrusive.js").is_vendored
        assert  self.blob("Scripts/jquery.validate.unobtrusive.min.js").is_vendored
        assert  self.blob("Scripts/jquery.unobtrusive-ajax.js").is_vendored
        assert  self.blob("Scripts/jquery.unobtrusive-ajax.min.js").is_vendored

        # NuGet Packages
        assert  self.blob("packages/Modernizr.2.0.6/Content/Scripts/modernizr-2.0.6-development-only.js").is_vendored

        # Test fixtures
        assert self.blob("test/fixtures/random.rkt").is_vendored
        assert self.blob("Test/fixtures/random.rkt").is_vendored

        # Cordova/PhoneGap
        assert self.blob("cordova.js").is_vendored
        assert self.blob("cordova.min.js").is_vendored
        assert self.blob("cordova-2.1.0.js").is_vendored
        assert self.blob("cordova-2.1.0.min.js").is_vendored

        # Vagrant
        assert self.blob("Vagrantfile").is_vendored

    def test_language(self):
        def _check_lang(sample):
            blob = self.blob(sample['path'])
            assert blob.language, 'No language for %s' % sample['path']
            assert sample['language'] == blob.language.name, blob.name

        Samples.each(_check_lang)

    def test_lexer(self):
        assert find_lexer_class('Ruby') == self.blob("Ruby/foo.rb").lexer

    def test_colorize(self):
        assert colorize == self.blob("Ruby/foo.rb").colorize()

    def test_colorize_does_skip_minified_files(self):
        assert None == self.blob("JavaScript/jquery-1.6.1.min.js").colorize()

    # Pygments.rb was taking exceeding long on this particular file
    def test_colorize_doesnt_blow_up_with_files_with_high_ratio_of_long_lines(self):
        assert None == self.blob("JavaScript/steelseries-min.js").colorize()


if __name__ == '__main__':
    main()
